""" Fixtures for data integration tests cases.
"""

from core_linked_records_app.components.pid_settings.models import PidSettings
from core_linked_records_app.system.pid_settings import (
    api as pid_settings_system_api,
)
from core_main_app.components.data.models import Data
from core_main_app.components.template.models import Template
from core_main_app.system import api as system_api
from core_main_app.utils.integration_tests.fixture_interface import (
    FixtureInterface,
)


class DataFixtures(FixtureInterface):
    """Data fixtures."""

    pid_settings: PidSettings = None
    template = None

    def insert_data(self):
        """Automatically inserts data."""
        self.create_pid_settings()
        self.insert_template()

    def create_pid_settings(self):
        """Creates PID settings"""
        self.pid_settings = PidSettings()
        pid_settings_system_api.upsert(self.pid_settings)

    def auto_set_pid(self, auto_set_pid_value: bool):
        """Change the value of the auto_set_pid field, enabling or disabling PID."""
        self.pid_settings.auto_set_pid = auto_set_pid_value
        pid_settings_system_api.upsert(self.pid_settings)

    def insert_record(self, data_name, data_pid, user):
        """Insert a record given a name, PID and user"""
        xml_content = (
            f"<mock><pid>{data_pid}</pid><name>{data_name}</name></mock>"
        )

        data = Data(
            template=self.template,
            user_id=user.id,
            title=data_name,
            xml_content=xml_content,
        )
        system_api.upsert_data(data)
        return data

    def insert_template(self):
        """Generate a unique Template."""
        self.template = Template()
        xsd = (
            '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
            '<xs:element name="mock">'
            "<xs:complexType>"
            "<xs:sequence>"
            '<xs:element name="pid"></xs:element>'
            '<xs:element name="name"></xs:element>'
            "</xs:sequence>"
            "</xs:complexType>"
            "</xs:element>"
            "</xs:schema>"
        )
        self.template.content = xsd
        self.template.hash = ""
        self.template.filename = "filename"
        self.template.save()



""" Unit tests for core_linked_records_app.system.data.api
"""

from unittest import TestCase
from unittest.mock import patch, Mock, MagicMock

from django.test import tag, override_settings

from core_linked_records_app.system.data import api as data_system_api
from core_linked_records_app.utils.providers import AbstractIdProvider
from core_main_app.commons.exceptions import DoesNotExist, ApiError
from core_main_app.components.data.models import Data


class TestIsPidDefinedForDataPsql(TestCase):
    """Test Is Pid Defined For Document"""

    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_wrong_document_id_raise_error(self, mock_data_get_by_id):
        """test_wrong_document_id_raise_error"""
        mock_data_get_by_id.side_effect = DoesNotExist("mock_does_not_exist")

        with self.assertRaises(Exception):
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )

    @override_settings(MONGODB_INDEXING=False)
    @patch("core_linked_records_app.system.data.api.Data.execute_query")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_undefined_pid_returns_false(
        self,
        mock_data_get_by_id,
        mock_get_pid_path_by_template,
        mock_data_execute_query,
    ):
        """test_undefined_pid_returns_false"""
        mock_data_get_by_id.return_value = MagicMock()
        mock_get_pid_path_by_template.return_value = MagicMock()
        mock_data_execute_query.return_value = []

        self.assertFalse(
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )
        )

    @override_settings(MONGODB_INDEXING=False)
    @patch("core_linked_records_app.system.data.api.Data.execute_query")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_duplicate_pid_returns_false(
        self,
        mock_data_get_by_id,
        mock_get_pid_path_by_template,
        mock_data_execute_query,
    ):
        """test_duplicate_pid_returns_false"""
        mock_data_get_by_id.return_value = MagicMock()
        mock_get_pid_path_by_template.return_value = MagicMock()
        mock_data_execute_query.return_value = [MagicMock(), MagicMock()]

        self.assertFalse(
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )
        )

    @override_settings(MONGODB_INDEXING=False)
    @patch("core_linked_records_app.system.data.api.Data.execute_query")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_defined_pid_for_other_document_returns_false(
        self,
        mock_data_get_by_id,
        mock_get_pid_path_by_template,
        mock_data_execute_query,
    ):
        """test_defined_pid_for_other_document_returns_false"""
        mock_data_get_by_id.return_value = MagicMock()
        mock_get_pid_path_by_template.return_value = MagicMock()
        mock_result = MagicMock()
        mock_result.pk = "mock_document_id_other"
        mock_data_execute_query.return_value = [mock_result]

        self.assertFalse(
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )
        )

    @override_settings(MONGODB_INDEXING=False)
    @patch("core_linked_records_app.system.data.api.Data.execute_query")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_defined_pid_for_current_document_returns_true(
        self,
        mock_data_get_by_id,
        mock_get_pid_path_by_template,
        mock_data_execute_query,
    ):
        """test_defined_pid_for_current_document_returns_true"""
        mock_data_get_by_id.return_value = MagicMock()
        mock_get_pid_path_by_template.return_value = MagicMock()
        mock_result = MagicMock()
        mock_result.pk = "mock_document_id"
        mock_data_execute_query.return_value = [mock_result]

        self.assertTrue(
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )
        )


class TestIsPidDefinedForDataMongo(TestCase):
    """Test Is Pid Defined For Document"""

    @tag("mongodb")
    @override_settings(MONGODB_INDEXING=True)
    @patch("core_main_app.components.mongo.models.MongoData.execute_query")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_undefined_pid_returns_false(
        self,
        mock_data_get_by_id,
        mock_get_pid_path_by_template,
        mock_data_execute_query,
    ):
        """test_undefined_pid_returns_false"""
        mock_data_get_by_id.return_value = MagicMock()
        mock_get_pid_path_by_template.return_value = MagicMock()
        mock_data_execute_query.return_value = []

        self.assertFalse(
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )
        )

    @tag("mongodb")
    @override_settings(MONGODB_INDEXING=True)
    @patch("core_main_app.components.mongo.models.MongoData.execute_query")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_duplicate_pid_returns_false(
        self,
        mock_data_get_by_id,
        mock_get_pid_path_by_template,
        mock_data_execute_query,
    ):
        """test_duplicate_pid_returns_false"""
        mock_data_get_by_id.return_value = MagicMock()
        mock_get_pid_path_by_template.return_value = MagicMock()
        mock_data_execute_query.return_value = [MagicMock(), MagicMock()]

        self.assertFalse(
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )
        )

    @tag("mongodb")
    @override_settings(MONGODB_INDEXING=True)
    @patch("core_main_app.components.mongo.models.MongoData.execute_query")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_defined_pid_for_other_document_returns_false(
        self,
        mock_data_get_by_id,
        mock_get_pid_path_by_template,
        mock_data_execute_query,
    ):
        """test_defined_pid_for_other_document_returns_false"""
        mock_data_get_by_id.return_value = MagicMock()
        mock_get_pid_path_by_template.return_value = MagicMock()
        mock_result = MagicMock()
        mock_result.pk = "mock_document_id_other"
        mock_data_execute_query.return_value = [mock_result]

        self.assertFalse(
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )
        )

    @tag("mongodb")
    @override_settings(MONGODB_INDEXING=True)
    @patch("core_main_app.components.mongo.models.MongoData.execute_query")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    @patch("core_linked_records_app.system.data.api.Data.get_by_id")
    def test_defined_pid_for_current_document_returns_true(
        self,
        mock_data_get_by_id,
        mock_get_pid_path_by_template,
        mock_data_execute_query,
    ):
        """test_defined_pid_for_current_document_returns_true"""
        mock_data_get_by_id.return_value = MagicMock()
        mock_get_pid_path_by_template.return_value = MagicMock()
        mock_result = MagicMock()
        mock_result.pk = "mock_document_id"
        mock_data_execute_query.return_value = [mock_result]

        self.assertTrue(
            data_system_api.is_pid_defined_for_data(
                "mock_pid", "mock_document_id"
            )
        )


class TestGetDataByPidPsql(TestCase):
    """Test Get Data By Pid"""

    @override_settings(MONGODB_INDEXING=False)
    @patch("core_linked_records_app.system.data.api.Data.execute_query")
    @patch("core_linked_records_app.system.data.api.PidPath.get_all")
    def test_query_returns_no_results_raises_error(
        self, mock_pid_path_get_all, mock_execute_query
    ):
        """test_query_returns_no_results_raises_error"""
        mock_pid_path_get_all.return_value = [MagicMock()]
        mock_queryset = MagicMock()
        mock_queryset.count.return_value = 0
        mock_execute_query.return_value = mock_queryset

        with self.assertRaises(DoesNotExist):
            data_system_api.get_data_by_pid("mock_pid")

    @override_settings(MONGODB_INDEXING=False)
    @patch("core_linked_records_app.system.data.api.Data.execute_query")
    @patch("core_linked_records_app.system.data.api.PidPath.get_all")
    def test_query_returns_several_results_raises_error(
        self, mock_pid_path_get_all, mock_execute_query
    ):
        """test_query_returns_several_results_raises_error"""
        mock_pid_path_get_all.return_value = [MagicMock()]
        mock_queryset = MagicMock()
        mock_queryset.count.return_value = 2
        mock_execute_query.return_value = mock_queryset

        with self.assertRaises(ApiError):
            data_system_api.get_data_by_pid("mock_pid")

    @override_settings(MONGODB_INDEXING=False)
    @patch("core_linked_records_app.system.data.api.Data.execute_query")
    @patch("core_linked_records_app.system.data.api.PidPath.get_all")
    def test_query_returns_single_result_returns_result(
        self, mock_pid_path_get_all, mock_execute_query
    ):
        """test_query_returns_single_result_returns_result"""
        mock_pid_path_get_all.return_value = [MagicMock()]
        mock_queryset = MagicMock()
        mock_queryset.count.return_value = 1
        mock_queryset.__getitem__.return_value = "mock_data"
        mock_execute_query.return_value = mock_queryset

        self.assertEqual(
            data_system_api.get_data_by_pid("mock_pid"), "mock_data"
        )


class TestGetDataByPidMongo(TestCase):
    """Test Get Data By Pid"""

    @tag("mongodb")
    @override_settings(MONGODB_INDEXING=True)
    @patch("core_main_app.components.mongo.models.MongoData.execute_query")
    @patch("core_linked_records_app.system.data.api.PidPath.get_all")
    def test_query_returns_no_results_raises_error(
        self, mock_pid_path_get_all, mock_execute_query
    ):
        """test_query_returns_no_results_raises_error"""
        mock_pid_path_get_all.return_value = [MagicMock()]
        mock_queryset = MagicMock()
        mock_queryset.count.return_value = 0
        mock_execute_query.return_value = mock_queryset

        with self.assertRaises(DoesNotExist):
            data_system_api.get_data_by_pid("mock_pid")

    @tag("mongodb")
    @override_settings(MONGODB_INDEXING=True)
    @patch("core_main_app.components.mongo.models.MongoData.execute_query")
    @patch("core_linked_records_app.system.data.api.PidPath.get_all")
    def test_query_returns_several_results_raises_error(
        self, mock_pid_path_get_all, mock_execute_query
    ):
        """test_query_returns_several_results_raises_error"""
        mock_pid_path_get_all.return_value = [MagicMock()]
        mock_queryset = MagicMock()
        mock_queryset.count.return_value = 2
        mock_execute_query.return_value = mock_queryset

        with self.assertRaises(ApiError):
            data_system_api.get_data_by_pid("mock_pid")

    @tag("mongodb")
    @override_settings(MONGODB_INDEXING=True)
    @patch("core_main_app.components.mongo.models.MongoData.execute_query")
    @patch("core_linked_records_app.system.data.api.PidPath.get_all")
    def test_query_returns_single_result_returns_result(
        self, mock_pid_path_get_all, mock_execute_query
    ):
        """test_query_returns_single_result_returns_result"""
        mock_pid_path_get_all.return_value = [MagicMock()]
        mock_queryset = MagicMock()
        mock_queryset.count.return_value = 1
        mock_queryset.__getitem__.return_value = "mock_data"
        mock_execute_query.return_value = mock_queryset

        self.assertEqual(
            data_system_api.get_data_by_pid("mock_pid"), "mock_data"
        )


class TestDeletePidForData(TestCase):
    """Test delete_pid_for_data"""

    @classmethod
    def setUpClass(cls) -> None:
        mock_data = Mock(spec=Data)
        mock_data.pk = "mock_pk"

        cls.mock_data = mock_data
        cls.mock_provider = Mock(spec=AbstractIdProvider)

    @patch.object(data_system_api, "delete_record_from_provider")
    @patch(
        "core_linked_records_app.system.data.api.get_value_from_dot_notation"
    )
    @patch("core_linked_records_app.system.data.api.xml_utils.raw_xml_to_dict")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    def test_get_dict_content_failure_calls_raw_xml_to_dict_utils(
        self,
        mock_get_pid_path_by_template,  # noqa, pylint: disable=unused-argument
        mock_raw_xml_to_dict,
        mock_get_value_from_dot_notation,
        mock_delete_record_from_provider,  # noqa, pylint: disable=unused-argument
    ):
        """test_get_dict_content_failure_calls_raw_xml_to_dict_utils"""
        self.mock_data.get_dict_content.side_effect = Exception(
            "mock_get_dict_content_exception"
        )
        mock_get_value_from_dot_notation.return_value = None

        data_system_api.delete_pid_for_data(self.mock_data)
        mock_raw_xml_to_dict.assert_called()

    @patch.object(data_system_api, "delete_record_from_provider")
    @patch(
        "core_linked_records_app.system.data.api.get_value_from_dot_notation"
    )
    @patch("core_linked_records_app.system.data.api.xml_utils.raw_xml_to_dict")
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    def test_raw_xml_to_dict_failure_raises_api_error(
        self,
        mock_get_pid_path_by_template,  # noqa, pylint: disable=unused-argument
        mock_raw_xml_to_dict,
        mock_get_value_from_dot_notation,
        mock_delete_record_from_provider,  # noqa, pylint: disable=unused-argument
    ):
        """test_raw_xml_to_dict_failure_raises_api_error"""
        self.mock_data.get_dict_content.side_effect = Exception(
            "mock_get_dict_content_exception"
        )
        mock_raw_xml_to_dict.side_effect = Exception(
            "mock_raw_xml_to_dict_exception"
        )
        mock_get_value_from_dot_notation.return_value = None

        with self.assertRaises(ApiError):
            data_system_api.delete_pid_for_data(self.mock_data)

    @patch.object(data_system_api, "delete_record_from_provider")
    @patch(
        "core_linked_records_app.system.data.api.get_value_from_dot_notation"
    )
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    def test_empty_pid_is_not_deleted(
        self,
        mock_get_pid_path_by_template,  # noqa, pylint: disable=unused-argument
        mock_get_value_from_dot_notation,
        mock_delete_record_from_provider,
    ):
        """test_empty_pid_is_not_deleted"""
        mock_get_value_from_dot_notation.return_value = None

        data_system_api.delete_pid_for_data(self.mock_data)
        mock_delete_record_from_provider.assert_not_called()

    @patch.object(data_system_api, "delete_record_from_provider")
    @patch(
        "core_linked_records_app.system.data.api.get_value_from_dot_notation"
    )
    @patch("core_linked_records_app.system.data.api.get_pid_path_by_template")
    def test_delete_pid_from_record_called_when_pid_exists(
        self,
        mock_get_pid_path_by_template,  # noqa, pylint: disable=unused-argument
        mock_get_value_from_dot_notation,
        mock_delete_record_from_provider,
    ):
        """test_delete_pid_from_record_called"""
        mock_pid = "mock_pid"
        mock_get_value_from_dot_notation.return_value = mock_pid

        data_system_api.delete_pid_for_data(self.mock_data)
        mock_delete_record_from_provider.assert_called_with(mock_pid)



""" Unit tests for data_html_user_renderer packages.
"""

from unittest import TestCase
from unittest.mock import Mock, patch

from rest_framework import status

from core_linked_records_app.rest.data.renderers import data_html_user_renderer
from tests.mocks import MockResponse


class TestDataHtmlUserRendererRender(TestCase):
    """Unit tests for `DataHtmlUserRenderer.render` method."""

    @patch.object(data_html_user_renderer, "render")
    def test_http_403_response_renders_error_page(self, mock_render):
        """test_http_403_response_renders_error_page"""
        mock_data = Mock()
        mock_response = MockResponse()
        mock_response.status_code = status.HTTP_403_FORBIDDEN
        mock_renderer_context = {
            "response": mock_response,
            "kwargs": {"record": "mock_record"},
        }

        renderer = data_html_user_renderer.DataHtmlUserRenderer()
        renderer.render(mock_data, renderer_context=mock_renderer_context)

        mock_render.assert_called_with(
            None,
            "core_main_app/common/commons/error.html",
            context={
                "error": "The user doesn't have enough rights to access "
                "document mock_record"
            },
        )



""" Unit tests for `core_linked_records_app.components.blob.watch` package.
"""

import json
from unittest import TestCase
from unittest.mock import patch, MagicMock

from django.urls import Resolver404
from rest_framework import status

from core_linked_records_app.components.blob import watch as blob_watch
from core_linked_records_app.components.pid_settings.models import PidSettings
from core_linked_records_app.system.blob import api as blob_system_api
from core_linked_records_app.utils import exceptions as pid_exceptions
from core_main_app.commons import exceptions as main_exceptions
from tests import mocks


class TestRegisterPidForBlobId(TestCase):
    """Unit tests for `_register_pid_for_blob_id` function."""

    def setUp(self):
        """setUp"""
        self.mock_kwargs = {
            "provider_name": MagicMock(),
            "pid_value": MagicMock(),
            "blob_id": MagicMock(),
        }

    @patch.object(blob_watch, "ProviderManager")
    def test_provider_manager_init_error_raises_invalid_provider(
        self, mock_provider_manager
    ):
        """test_provider_manager_init_error_raises_invalid_provider"""
        mock_provider_manager.side_effect = Exception(
            "mock_provider_manager_exception"
        )

        with self.assertRaises(pid_exceptions.InvalidProviderError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

    @patch.object(blob_watch, "ProviderManager")
    def test_provider_manager_get_error_raises_invalid_provider(
        self, mock_provider_manager
    ):
        """test_provider_manager_get_error_raises_invalid_provider"""
        mock_provider_manager_object = MagicMock()
        mock_provider_manager_object.get.side_effect = Exception(
            "mock_provider_manager_get_exception"
        )
        mock_provider_manager.return_value = mock_provider_manager_object

        with self.assertRaises(pid_exceptions.InvalidProviderError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

    @patch.object(blob_watch, "ProviderManager")
    def test_pid_value_lookup_url_replaced_by_local_url(
        self, mock_provider_manager
    ):
        """test_pid_value_lookup_url_replaced_by_local_url"""
        mock_provider_manager_object = MagicMock()
        mock_provider = MagicMock()
        mock_provider_manager_object.get.return_value = mock_provider
        mock_provider_manager.return_value = mock_provider_manager_object

        with self.assertRaises(pid_exceptions.PidResolverError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

        self.mock_kwargs["pid_value"].replace.assert_called_with(
            mock_provider.provider_lookup_url, mock_provider.local_url
        )

    @patch.object(blob_watch, "resolve")
    @patch.object(blob_watch, "ProviderManager")
    def test_resolver_error_raises_pid_resolver_error(
        self,
        mock_provider_manager,  # noqa, pylint: disable=unused-argument
        mock_resolve,
    ):
        """test_resolver_error_raises_pid_resolver_error"""
        mock_resolve.side_effect = Resolver404()

        with self.assertRaises(pid_exceptions.PidResolverError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

    @patch.object(blob_watch, "resolve")
    @patch.object(blob_watch, "ProviderManager")
    def test_invalid_view_name_raises_pid_resolver_error(
        self,
        mock_provider_manager,  # noqa, pylint: disable=unused-argument
        mock_resolve,
    ):
        """test_invalid_view_name_raises_pid_resolver_error"""
        mock_resolver_match = MagicMock()
        mock_resolver_match.view_name = "invalid_view_name"
        mock_resolve.return_value = mock_resolver_match

        with self.assertRaises(pid_exceptions.PidResolverError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

    @patch.object(blob_watch, "split_prefix_from_record")
    @patch.object(blob_watch, "resolve")
    @patch.object(blob_watch, "ProviderManager")
    def test_split_prefix_from_record_invalid_prefix_raises_invalid_pid_error(
        self,
        mock_provider_manager,  # noqa, pylint: disable=unused-argument
        mock_resolve,
        mock_split_prefix_from_record,
    ):
        """test_split_prefix_from_record_invalid_prefix_raises_invalid_pid_error"""
        mock_resolver_match = MagicMock()
        mock_resolver_match.view_name = "core_linked_records_provider_record"
        mock_resolve.return_value = mock_resolver_match

        mock_split_prefix_from_record.side_effect = (
            pid_exceptions.InvalidPrefixError(
                "mock_split_prefix_from_record_invalid_prefix_error"
            )
        )

        with self.assertRaises(pid_exceptions.InvalidPidError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

    @patch.object(blob_watch, "split_prefix_from_record")
    @patch.object(blob_watch, "resolve")
    @patch.object(blob_watch, "ProviderManager")
    def test_split_prefix_from_record_invalid_record_raises_invalid_pid_error(
        self,
        mock_provider_manager,  # noqa, pylint: disable=unused-argument
        mock_resolve,
        mock_split_prefix_from_record,
    ):
        """test_split_prefix_from_record_invalid_record_raises_invalid_pid_error"""
        mock_resolver_match = MagicMock()
        mock_resolver_match.view_name = "core_linked_records_provider_record"
        mock_resolve.return_value = mock_resolver_match

        mock_split_prefix_from_record.side_effect = (
            pid_exceptions.InvalidRecordError(
                "mock_split_prefix_from_record_invalid_record_error"
            )
        )

        with self.assertRaises(pid_exceptions.InvalidPidError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

    @patch.object(blob_watch, "split_prefix_from_record")
    @patch.object(blob_watch, "resolve")
    @patch.object(blob_watch, "ProviderManager")
    def test_provider_create_called(
        self,
        mock_provider_manager,
        mock_resolve,
        mock_split_prefix_from_record,
    ):
        """test_provider_create_called"""
        mock_provider_manager_object = MagicMock()
        mock_provider = MagicMock()
        mock_provider_manager_object.get.return_value = mock_provider
        mock_provider_manager.return_value = mock_provider_manager_object
        mock_resolver_match = MagicMock()
        mock_resolver_match.view_name = "core_linked_records_provider_record"
        mock_resolve.return_value = mock_resolver_match

        mock_prefix = MagicMock()
        mock_record = MagicMock()
        mock_split_prefix_from_record.return_value = mock_prefix, mock_record

        with self.assertRaises(pid_exceptions.PidCreateError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

        mock_provider.create.assert_called_with(mock_prefix, mock_record)

    @patch.object(blob_watch, "blob_system_api")
    @patch.object(blob_watch, "split_prefix_from_record")
    @patch.object(blob_watch, "resolve")
    @patch.object(blob_watch, "ProviderManager")
    def test_invalid_provider_response_raises_pid_create_error(
        self,
        mock_provider_manager,
        mock_resolve,
        mock_split_prefix_from_record,
        mock_blob_system_api,
    ):
        """test_invalid_provider_response_raises_pid_create_error_with_content"""
        mock_provider_manager_object = MagicMock()
        mock_provider = MagicMock()
        mock_provider_manager_object.get.return_value = mock_provider
        mock_provider_manager.return_value = mock_provider_manager_object
        mock_resolver_match = MagicMock()
        mock_resolver_match.view_name = "core_linked_records_provider_record"
        mock_resolve.return_value = mock_resolver_match

        mock_split_prefix_from_record.return_value = MagicMock(), MagicMock()

        mock_provider_response = MagicMock()
        mock_provider_response.status_code = status.HTTP_400_BAD_REQUEST
        mock_provider_response.content = json.dumps(
            {"message": "mock_error_message"}
        )
        mock_provider.create.return_value = mock_provider_response

        mock_blob_system_api.get_blob_by_pid.return_value = MagicMock()

        with self.assertRaises(pid_exceptions.PidCreateError):
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs)

    @patch.object(blob_watch, "blob_system_api")
    @patch.object(blob_watch, "split_prefix_from_record")
    @patch.object(blob_watch, "resolve")
    @patch.object(blob_watch, "ProviderManager")
    def test_succesful_execution_returns_provider_response_url(
        self,
        mock_provider_manager,
        mock_resolve,
        mock_split_prefix_from_record,
        mock_blob_system_api,  # noqa, pylint: disable=unused-argument
    ):
        """test_succesful_execution_returns_provider_response_url"""
        mock_provider_manager_object = MagicMock()
        mock_provider = MagicMock()
        mock_provider_manager_object.get.return_value = mock_provider
        mock_provider_manager.return_value = mock_provider_manager_object
        mock_resolver_match = MagicMock()
        mock_resolver_match.view_name = "core_linked_records_provider_record"
        mock_resolve.return_value = mock_resolver_match

        mock_split_prefix_from_record.return_value = MagicMock(), MagicMock()

        mock_provider_response = MagicMock()
        mock_provider_response.status_code = status.HTTP_200_OK
        mock_provider_content = {"url": "mock_url"}
        mock_provider_response.content = json.dumps(mock_provider_content)
        mock_provider.create.return_value = mock_provider_response

        self.assertEqual(
            blob_watch._register_pid_for_blob_id(**self.mock_kwargs),
            mock_provider_content["url"],
        )


class TestSetBlobPid(TestCase):
    """Test Set Blob Pid"""

    def setUp(self):
        """setUp"""
        self.mock_document = mocks.MockDocument()

    @patch.object(blob_watch, "transaction")
    def test_transaction_on_commit_is_called(self, mock_transaction):
        """test_transaction_on_commit_is_called"""
        blob_watch.set_blob_pid(None, self.mock_document)
        mock_transaction.on_commit.assert_called()

    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(PidSettings, "get")
    def test_pid_setting_get_failure_raises_core_error(
        self, mock_get, mock_get_pid_for_blob
    ):
        """test_pid_setting_get_failure_raises_core_error"""

        mock_get_pid_for_blob.side_effect = main_exceptions.DoesNotExist(
            "pid_does_not_exist"
        )
        mock_get.side_effect = Exception("mock_get_exception")

        with self.assertRaises(main_exceptions.CoreError):
            blob_watch._set_blob_pid(self.mock_document)

    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(PidSettings, "get")
    def test_get_pid_for_blob_failure_raises_core_error(
        self, mock_get, mock_get_pid_for_blob
    ):
        """test_get_pid_for_blob_failure_raises_core_error"""
        mock_get.return_value = MagicMock()
        mock_get_pid_for_blob.side_effect = Exception(
            "mock_get_pid_for_blob_exception"
        )

        with self.assertRaises(main_exceptions.CoreError):
            blob_watch._set_blob_pid(self.mock_document)

    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(blob_watch, "_register_pid_for_blob_id")
    @patch.object(PidSettings, "get")
    def test_register_pid_for_blob_failure_raises_core_error(
        self,
        mock_get,
        mock_register_pid_for_blob_id,
        mock_get_pid_for_blob,
    ):
        """test_register_pid_for_blob_failure_raises_core_error"""

        mock_get.return_value = mocks.MockPidSettings()
        mock_get_pid_for_blob.side_effect = main_exceptions.DoesNotExist(
            "pid_does_not_exist"
        )
        mock_register_pid_for_blob_id.side_effect = Exception(
            "mock_register_pid_for_blob_id_exception"
        )

        with self.assertRaises(main_exceptions.CoreError):
            blob_watch._set_blob_pid(self.mock_document)

    @patch.object(blob_system_api, "set_pid_for_blob")
    @patch.object(blob_watch, "_register_pid_for_blob_id")
    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(PidSettings, "get")
    def test_set_pid_for_blob_failure_raises_core_error(
        self,
        mock_get,
        mock_get_pid_for_blob,
        mock_register_pid_for_blob_id,
        mock_set_pid_for_blob,
    ):
        """test_set_pid_for_blob_failure_raises_core_error"""

        mock_get_pid_for_blob.side_effect = main_exceptions.DoesNotExist(
            "pid_does_not_exist"
        )
        mock_register_pid_for_blob_id.return_value = MagicMock()
        mock_get.return_value = mocks.MockPidSettings()
        mock_set_pid_for_blob.side_effect = Exception(
            "mock_set_pid_for_blob_exception"
        )

        with self.assertRaises(main_exceptions.CoreError):
            blob_watch._set_blob_pid(self.mock_document)

    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(blob_system_api, "set_pid_for_blob")
    @patch.object(PidSettings, "get")
    def test_auto_set_pid_false_does_not_assign_pid(
        self, mock_get, mock_set_pid_for_blob, mock_get_pid_for_blob
    ):
        """test_auto_set_pid_false_does_not_assign_pid"""

        mock_get_pid_for_blob.side_effect = main_exceptions.DoesNotExist(
            "pid_does_not_exist"
        )
        mock_pid_settings = mocks.MockPidSettings()
        mock_pid_settings.auto_set_pid = False
        mock_get.return_value = mock_pid_settings

        blob_watch._set_blob_pid(self.mock_document)
        mock_set_pid_for_blob.assert_not_called()

    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(blob_system_api, "set_pid_for_blob")
    @patch.object(PidSettings, "get")
    def test_auto_set_pid_true_assign_pid(
        self,
        mock_get,
        mock_set_pid_for_blob,
        mock_get_pid_for_blob,
    ):
        """test_auto_set_pid_true_assign_pid"""

        mock_get_pid_for_blob.side_effect = main_exceptions.DoesNotExist(
            "pid_does_not_exist"
        )  # noqa
        mock_get.return_value = mocks.MockPidSettings()
        mock_set_pid_for_blob.return_value = None

        blob_watch._set_blob_pid(self.mock_document)
        mock_set_pid_for_blob.assert_called_once()

    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(PidSettings, "get")
    def test_auto_set_pid_false_returns_none(
        self, mock_get, mock_get_pid_for_blob
    ):
        """test_auto_set_pid_false_returns_none"""

        mock_get_pid_for_blob.side_effect = main_exceptions.DoesNotExist(
            "pid_does_not_exist"
        )
        mock_pid_settings = mocks.MockPidSettings()
        mock_pid_settings.auto_set_pid = False
        mock_get.return_value = mock_pid_settings

        self.assertIsNone(blob_watch._set_blob_pid(self.mock_document))

    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(blob_system_api, "set_pid_for_blob")
    @patch.object(PidSettings, "get")
    def test_auto_set_pid_true_returns_none(
        self,
        mock_get,
        mock_set_pid_for_blob,
        mock_get_pid_for_blob,
    ):
        """test_auto_set_pid_true_returns_none"""

        mock_get_pid_for_blob.side_effect = main_exceptions.DoesNotExist(
            "pid_does_not_exist"
        )  # noqa
        mock_get.return_value = mocks.MockPidSettings()
        mock_set_pid_for_blob.return_value = None

        self.assertIsNone(blob_watch._set_blob_pid(self.mock_document))

    @patch.object(PidSettings, "get")
    @patch.object(blob_system_api, "get_pid_for_blob")
    @patch.object(blob_system_api, "set_pid_for_blob")
    def test_pid_not_created_if_already_assigned(
        self, mock_set_pid_for_blob, mock_get_pid_for_blob, mock_settings_get
    ):
        """test_pid_not_created_if_already_assigned"""

        mock_get_pid_for_blob.return_value = "mock_pid"
        mock_settings_get.return_value = mocks.MockPidSettings()

        blob_watch._set_blob_pid(self.mock_document)
        mock_set_pid_for_blob.assert_not_called()


class TestDeleteBlobPid(TestCase):
    """Unit tests for detele_blob_pid function."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_blob = mocks.MockDocument()

    @patch.object(blob_system_api, "delete_pid_for_blob")
    def test_delete_pid_for_blob_is_called(self, mock_delete_pid_for_blob):
        """test_delete_pid_for_blob_is_called"""
        blob_watch.delete_blob_pid(None, self.mock_blob)

        mock_delete_pid_for_blob.assert_called_with(self.mock_blob)

    @patch.object(blob_watch, "logger")
    @patch.object(blob_system_api, "delete_pid_for_blob")
    def test_delete_pid_for_blob_error_raise_warning(
        self, mock_delete_pid_for_blob, mock_logger
    ):
        """test_delete_pid_for_blob_error_raise_warning"""
        mock_delete_pid_for_blob.side_effect = Exception(
            "mock_delete_pid_for_blob_exception"
        )
        blob_watch.delete_blob_pid(None, self.mock_blob)

        mock_logger.warning.assert_called()



""" Migration to alter PidXpath validation.

Generated by Django 3.2.18 on 2023-03-27
"""

from django.db import migrations, models

import core_linked_records_app.utils.dict


class Migration(migrations.Migration):
    """Migration class."""

    dependencies = [
        ("core_linked_records_app", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="pidxpath",
            name="xpath",
            field=models.CharField(
                max_length=255,
                validators=[
                    core_linked_records_app.utils.dict.validate_dot_notation
                ],
            ),
        ),
    ]



""" Data HTML renderer for django REST API
"""

import logging

from rest_framework import renderers, status
from rest_framework.exceptions import APIException

from core_main_app.components.data import api as data_api
from core_main_app.utils.rendering import render
from core_main_app.utils.view_builders import data as data_view_builder

logger = logging.getLogger(__name__)


class DataHtmlUserRenderer(renderers.BaseRenderer):
    """Data Html User Renderer"""

    media_type = "text/html"
    format = "html"
    charset = "utf-8"

    def render(self, data, media_type=None, renderer_context=None):
        """Render the data object by returning the user template

        Args:
            data:
            media_type:
            renderer_context:

        Returns: html page
        """
        # Build the request object or set it up to None if undefined
        request = (
            renderer_context["request"]
            if "request" in renderer_context
            else None
        )

        # If the access to the data is forbidden.
        if (
            "response" in renderer_context
            and renderer_context["response"].status_code == 403
        ):
            error_msg = (
                "The user doesn't have enough rights to access document %s"
                % renderer_context["kwargs"]["record"]
            )
            return render(
                request,
                "core_main_app/common/commons/error.html",
                context={"error": error_msg},
            )

        # If the data retrieved contains an error
        if "status" in data and data["status"] == "error":
            return render(
                request,
                "core_main_app/common/commons/error.html",
                context={"error": data["message"]},
            )

        try:
            # Check the renderer format
            if (
                request
                and request.query_params != {}
                and "format" in request.query_params
                and request.query_params["format"] != "html"
            ):
                raise APIException(
                    "Wrong data format parameter.", status.HTTP_404_NOT_FOUND
                )

            data_object = data_api.get_by_id(data["id"], request.user)
            page_context = data_view_builder.build_page(
                data_object, display_download_options=True
            )

            return data_view_builder.render_page(
                request,
                render,
                "core_main_app/user/data/detail.html",
                page_context,
            )
        except APIException as api_error:
            return render(
                request,
                "core_main_app/common/commons/error.html",
                context={"error": str(api_error)},
            )
        except Exception as exception:
            logger.error("Error while building data page: %s", str(exception))

            if (
                "kwargs" in renderer_context
                and "record" in renderer_context["kwargs"]
            ):
                error_msg = (
                    "Document %s does not exist."
                    % renderer_context["kwargs"]["record"]
                )
            else:
                error_msg = "Invalid request provided."

            return render(
                request,
                "core_main_app/common/commons/error.html",
                context={"error": error_msg},
            )



""" System API to manage LocalId objects.
"""

import logging

from core_linked_records_app.components.local_id.models import LocalId
from core_main_app.commons import exceptions

logger = logging.getLogger(__name__)


def get_by_name(record_name):
    """Retrieve the record by name.

    Args:
        record_name:

    Returns:
    """
    try:
        return LocalId.get_by_name(record_name)
    except exceptions.DoesNotExist as dne:
        raise exceptions.DoesNotExist(str(dne))
    except Exception as exc:
        error_message = (
            "An unexpected error occurred while retrieving LocalId by name"
        )

        logger.error("%s: %s", error_message, str(exc))
        raise exceptions.ApiError(f"{error_message}.")


def get_by_class_and_id(record_object_class, record_object_id):
    """Retrieve LocalID using linked object class and ID

    Args:
        record_object_class:
        record_object_id:

    Returns:
    """
    try:
        return LocalId.get_by_class_and_id(
            record_object_class, record_object_id
        )
    except exceptions.DoesNotExist as dne:
        raise exceptions.DoesNotExist(str(dne))
    except Exception as exc:
        error_message = "An unexpected error occurred while retrieving LocalId by class and id"

        logger.error("%s: %s", error_message, str(exc))
        raise exceptions.ApiError(f"{error_message}.")


def insert(local_id_object):
    """Insert the record in the collection.

    Args:
        local_id_object:

    Returns:
    """
    try:
        LocalId.upsert(local_id_object)
        return local_id_object
    except Exception as exc:
        error_message = "An unexpected error occurred while inserting LocalId"

        logger.error("%s: %s", error_message, str(exc))
        raise exceptions.ApiError(f"{error_message}.")


def delete(local_id_object):
    """Delete the record.

    Args:
        local_id_object:

    Returns:

    """
    try:
        return local_id_object.delete()
    except Exception as exc:
        error_message = "An unexpected error occurred while deleting LocalId"

        logger.error("%s: %s", error_message, str(exc))
        raise exceptions.ApiError(f"{error_message}.")



""" Migration to change PidXpath designation into PidPath.

Generated by Django 4.2.3 on 2023-11-14
"""

from django.db import migrations


class Migration(migrations.Migration):
    """Migration class."""

    dependencies = [
        ("core_linked_records_app", "0003_create_permissions"),
    ]

    operations = [
        migrations.RenameModel("PidXpath", "PidPath"),
        migrations.RenameField("PidPath", "xpath", "path"),
    ]



""" Rest API views to retrieve PID settings
"""

from rest_framework import status
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from core_linked_records_app.components.pid_settings import (
    api as pid_settings_api,
)
from core_linked_records_app.rest.pid_settings.serializers import (
    PidSettingsSerializer,
)
from core_linked_records_app.utils.pid import get_pid_settings_dict


class PidSettingsView(APIView):
    """Retrieve PID settings"""

    permission_classes = (IsAuthenticated,)

    def get(self, request):
        """Retrieve the settings for the PID system

        Args:
            request:

        Returns:
        """
        try:
            pid_settings = pid_settings_api.get(request.user)
            return Response(
                get_pid_settings_dict(pid_settings),
                status=status.HTTP_200_OK,
            )
        except Exception as exc:
            return Response(
                {
                    "message": f"An unexpected error occurred while displaying "
                    f"PidSettings: {str(exc)}"
                },
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )

    def patch(self, request):
        """Update settings for the PID system. Current, only works for automatically
        setting PIDs (`auto_set_pid`).

        Args:
            request:

        Returns:
        """
        if not request.user.is_superuser:
            return Response(
                {"message": "Only a superuser can use this feature."},
                status=status.HTTP_403_FORBIDDEN,
            )

        try:
            pid_settings_serializer = PidSettingsSerializer(
                data=request.data, context={"request": request}
            )

            if not pid_settings_serializer.is_valid():
                return Response(
                    {"message": "Invalid data provided"},
                    status=status.HTTP_400_BAD_REQUEST,
                )

            pid_settings_serializer.update(
                pid_settings_api.get(request.user),
                pid_settings_serializer.validated_data,
            )

            return self.get(request)
        except Exception as exc:
            return Response(
                {
                    "message": f"An unexpected error occurred while modifying "
                    f"PidSettings: {str(exc)}"
                },
                status=status.HTTP_500_INTERNAL_SERVER_ERROR,
            )



""" Unit tests for core_linked_records_app.utils.providers.__init__.
"""

from unittest import TestCase
from unittest.mock import patch, Mock

from core_linked_records_app.utils import providers
from core_main_app.commons.exceptions import CoreError
from tests.mocks import MockResponse


class TestDeleteRecordFromProvider(TestCase):
    """Unit tests for `delete_record_from_provider` function."""

    @patch.object(providers, "ProviderManager")
    def test_provider_delete_is_called(self, mock_provider_manager):
        """test_provider_delete_is_called"""
        mock_provider = Mock()
        mock_record = "mock_record"

        mock_provider_manager().get.return_value = mock_provider
        mock_provider.delete.return_value = MockResponse()

        providers.delete_record_from_provider(mock_record)
        mock_provider.delete.assert_called_with(mock_record)

    @patch.object(providers, "logger")
    @patch.object(providers, "ProviderManager")
    def test_provider_delete_failure_raises_api_error(
        self,
        mock_provider_manager,
        mock_logger,  # noqa, pylint: disable=unused-argument
    ):
        """test_provider_delete_failure_is_logged"""
        mock_provider = Mock()
        mock_provider.delete.return_value = MockResponse(status_code=500)
        mock_record = "mock_record"

        mock_provider_manager().get.return_value = mock_provider

        with self.assertRaises(CoreError):
            providers.delete_record_from_provider(mock_record)



""" Access control helpers for PidPath APIs.
"""

from core_main_app.components.template.access_control import (
    check_can_read_template,
)


def can_get_by_template(func, template, user):
    """Check that user can use the `get_by_template` function.

    Args:
        func:
        template:
        user:

    Returns:
    """
    # Check can read template and PidPath
    check_can_read_template(template, user)
    return func(template, user)



""" Access control methods for `core_linked_records.components.data.api`.
"""

from core_main_app.access_control.api import check_can_read_document
from core_main_app.components.data.models import Data


def can_get_data_by_pid(func, pid, request):
    """Access control for the `get_data_by_pid` function.

    Args:
        func:
        pid:
        request:

    Returns:
    """
    data = func(pid, request)

    if not request.user.is_superuser:
        check_can_read_document(data, request.user)

    return data


def can_get_pid_for_data(func, data_id, request):
    """Access control for the `get_pid_for_data` function.

    Args:sel
        func:
        data_id:
        request:

    Returns:
    """
    if not request.user.is_superuser:
        check_can_read_document(Data.get_by_id(data_id), request.user)
    return func(data_id, request)



""" REST views for PidPath collection
"""

from rest_framework.generics import (
    ListCreateAPIView,
    RetrieveUpdateDestroyAPIView,
)
from rest_framework.permissions import IsAuthenticated

from core_linked_records_app.components.pid_path.models import PidPath
from core_linked_records_app.rest.pid_path.serializers import (
    PidPathSerializer,
)


class PidPathListView(ListCreateAPIView):
    """View for listing and creating `PidPath` objects"""

    permission_classes = (IsAuthenticated,)

    queryset = PidPath.objects.all()
    serializer_class = PidPathSerializer


class PidPathDetailView(RetrieveUpdateDestroyAPIView):
    """View for listing and updating a single `PidPath` object"""

    permission_classes = (IsAuthenticated,)

    queryset = PidPath.objects.all()
    serializer_class = PidPathSerializer



""" ACL tests for `core_linked_records.components.pid_settings.api`.
"""

from unittest import TestCase
from unittest.mock import patch, MagicMock

from core_linked_records_app.components.pid_settings import (
    api as pid_settings_api,
    access_control as pid_settings_acl,
)
from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.tests_tools.MockUser import create_mock_user


class TestGet(TestCase):
    """ACL tests for `get` function."""

    def setUp(self) -> None:
        """setUp"""
        self.mock_pid_settings = MagicMock()

    @patch.object(pid_settings_api, "pid_settings_system_api")
    def test_superuser_can_access(self, mock_pid_settings_system_api):
        """test_superuser_can_access"""
        user = create_mock_user("1", is_superuser=True)
        mock_pid_settings_system_api.get.return_value = self.mock_pid_settings

        self.assertEqual(pid_settings_api.get(user), self.mock_pid_settings)

    @patch.object(pid_settings_api, "pid_settings_system_api")
    @patch.object(pid_settings_acl, "check_has_perm")
    def test_registered_user_without_perm_cannot_access(
        self, mock_check_has_perm, mock_pid_settings_system_api
    ):
        """test_registered_user_without_perm_cannot_access"""
        user = create_mock_user("1")
        mock_check_has_perm.side_effect = AccessControlError(
            "mock_check_has_perm_acl_error"
        )
        mock_pid_settings_system_api.get.return_value = self.mock_pid_settings

        with self.assertRaises(AccessControlError):
            pid_settings_api.get(user)

    @patch.object(pid_settings_api, "pid_settings_system_api")
    @patch.object(pid_settings_acl, "check_has_perm")
    def test_registered_user_with_perm_can_access(
        self,
        mock_check_has_perm,  # noqa, pylint: disable=unused-argument
        mock_pid_settings_system_api,
    ):
        """test_registered_user_with_perm_can_access"""
        user = create_mock_user("1")
        mock_pid_settings_system_api.get.return_value = self.mock_pid_settings

        self.assertEqual(pid_settings_api.get(user), self.mock_pid_settings)

    @patch.object(pid_settings_api, "pid_settings_system_api")
    @patch.object(pid_settings_acl, "check_has_perm")
    def test_anonymous_user_without_perm_cannot_access(
        self, mock_check_has_perm, mock_pid_settings_system_api
    ):
        """test_anonymous_user_without_perm_cannot_access"""
        user = create_mock_user("1", is_anonymous=True)
        mock_check_has_perm.side_effect = AccessControlError(
            "mock_check_has_perm_acl_error"
        )
        mock_pid_settings_system_api.get.return_value = self.mock_pid_settings

        with self.assertRaises(AccessControlError):
            pid_settings_api.get(user)

    @patch.object(pid_settings_api, "pid_settings_system_api")
    @patch.object(pid_settings_acl, "check_has_perm")
    def test_anonymous_user_with_perm_can_access(
        self,
        mock_check_has_perm,  # noqa, pylint: disable=unused-argument
        mock_pid_settings_system_api,
    ):
        """test_anonymous_user_with_perm_can_access"""
        user = create_mock_user("1", is_anonymous=True)
        mock_pid_settings_system_api.get.return_value = self.mock_pid_settings

        self.assertEqual(pid_settings_api.get(user), self.mock_pid_settings)



""" System API to manage PidPath objects.
"""

import logging

from core_linked_records_app import settings
from core_linked_records_app.components.pid_path.models import PidPath

logger = logging.getLogger(__name__)


def get_pid_path_by_template(template):
    """Retrieve path associated with a specific template ID

    Args:
        template: Template object

    Returns:
        PidPath - PidPath object, linking template ID and path
    """
    pid_path_object = PidPath.get_by_template(template)

    if pid_path_object is None:
        return PidPath(template=template, path=settings.PID_PATH)

    return pid_path_object



""" System API to manage Data objects.
"""

import logging

from django.conf import settings as conf_settings

from core_linked_records_app import settings
from core_linked_records_app.components.pid_path.models import PidPath
from core_linked_records_app.system.pid_path.api import (
    get_pid_path_by_template,
)
from core_linked_records_app.utils.dict import get_value_from_dot_notation
from core_linked_records_app.utils.providers import delete_record_from_provider
from core_main_app.commons.exceptions import DoesNotExist, ApiError
from core_main_app.components.data.models import Data
from core_main_app.utils import xml as xml_utils
from core_main_app.utils.query.mongo.prepare import sanitize_value

logger = logging.getLogger(__name__)


def is_pid_defined_for_data(pid, document_id):
    """Determine if a given PID match the document ID provided.

    Params:
        pid:
        document_id:

    Returns:
    """
    data = Data.get_by_id(document_id)

    pid_path_object = get_pid_path_by_template(data.template)
    pid_path = pid_path_object.path
    query = {
        f"dict_content__{pid_path.replace('.', '__')}__exact": sanitize_value(
            pid
        )
    }

    # pylint: disable=import-outside-toplevel
    if conf_settings.MONGODB_INDEXING:
        from mongoengine.queryset.visitor import Q  # noqa
        from core_main_app.components.mongo.models import MongoData

        query_result = MongoData.execute_query(
            Q(**query),
            order_by_field=[],
        )
    else:
        from django.db.models import (
            Q,
        )

        query_result = Data.execute_query(
            Q(**query),
            order_by_field=[],
        )

    return len(query_result) == 1 and query_result[0].pk == document_id


def is_pid_defined(pid):
    """Determine if a given PID already exists.

    Params:
        pid:

    Returns:
    """
    try:
        get_data_by_pid(pid)
        return True
    except (DoesNotExist, ApiError):
        return False


def delete_pid_for_data(data: Data):
    """Deletes the PID assigned to the data passed in parameter. If no PID has
    been assigned, the function simply exits.

    Args:
        data: Data - The data for which the PID needs to be deleted.
    """
    # Retrieve the PID_PATH associated with the data template.
    pid_path_object = get_pid_path_by_template(data.template)
    pid_path = pid_path_object.path

    # Retrieve the data dict content to search for the PID_PATH using dot notation.
    try:  # Try to retrieve the dict content using data model
        dict_content = data.get_dict_content()
    except Exception:  # noqa, pylint: disable=broad-except
        # Retrieving the dict content is not possible, convert the xml to dict
        try:
            dict_content = xml_utils.raw_xml_to_dict(data.xml_content)
        except (
            Exception
        ) as exc:  # Converting the xml to dict is not possible either
            raise ApiError(
                f"Impossible to retrieve the dict content for the data: {str(exc)}"
            ) from exc

    # Return PID value from the document and the PID_PATH
    current_pid = get_value_from_dot_notation(
        dict_content,
        pid_path,
    )

    if not current_pid:  # If there is no previous PID assigned.
        logger.info("No PID assigned to the data %s", str(data.pk))
        return

    # From the PID url (e.g. https://pid-system.org/prefix/record), retrieve
    # only the prefix and record (e.g. prefix/record) stored in DB.
    pid_internal_name = "/".join(current_pid.split("/")[-2:])

    # Delete the PID using the internal name.
    delete_record_from_provider(pid_internal_name)


def get_data_by_pid(pid):
    """Return data object with the given pid.

    Parameters:
        pid:

    Returns: data object
    """
    # pylint: disable=import-outside-toplevel
    if conf_settings.MONGODB_INDEXING:
        from mongoengine.queryset.visitor import Q  # noqa
    else:
        from django.db.models import Q

    pid_path_query = Q(
        **{
            f"dict_content__{settings.PID_PATH.replace('.', '__')}__exact": sanitize_value(
                pid
            )
        }
    )

    # Create the or query with all the different PID path available.
    for pid_path_object in PidPath.get_all():
        pid_path_query |= Q(
            **{
                f"dict_content__{pid_path_object.path.replace('.', '__')}__exact": sanitize_value(
                    pid
                )
            }
        )
    # Execute built query and retrieve number of items returned.
    if conf_settings.MONGODB_INDEXING:
        from core_main_app.components.mongo.models import MongoData

        query_result = MongoData.execute_query(
            pid_path_query,
            order_by_field=[],
        )
    else:
        query_result = Data.execute_query(
            pid_path_query,
            order_by_field=[],
        )

    query_result_length = query_result.count()

    if query_result_length == 0:
        raise DoesNotExist("PID is not attached to any data.")
    if query_result_length != 1:
        raise ApiError("PID must be unique.")

    return query_result[0]



""" Unit tests for core_linked_records_app.components.pid_settings.watch
"""

from unittest import TestCase
from unittest.mock import patch

from core_linked_records_app.components.pid_settings import (
    watch as pid_settings_watch,
)
from core_linked_records_app.components.pid_settings.models import PidSettings
from core_linked_records_app.system.pid_settings import (
    api as pid_settings_system_api,
)


class TestInit(TestCase):
    """Test Init"""

    @patch.object(PidSettings, "get")
    def test_pid_settings_get_failure_returns_none(
        self, mock_pid_settings_get
    ):
        """test_pid_settings_get_failure_returns_none"""

        mock_pid_settings_get.side_effect = Exception(
            "mock_pid_settings_get_exception"
        )

        self.assertIsNone(pid_settings_watch.init())

    @patch.object(PidSettings, "get")
    def test_pid_settings_get_true_returns_none(self, mock_pid_settings_get):
        """test_pid_settings_get_true_returns_none"""

        mock_pid_settings_get.return_value = True

        self.assertIsNone(pid_settings_watch.init())

    @patch.object(PidSettings, "__init__")
    @patch.object(PidSettings, "get")
    def test_pid_settings_init_failure_returns_none(
        self, mock_pid_settings_get, mock_pid_settings_init
    ):
        """test_pid_settings_init_failure_returns_none"""

        mock_pid_settings_get.return_value = False
        mock_pid_settings_init.side_effect = Exception(
            "mock_pid_settings_init_exception"
        )

        self.assertIsNone(pid_settings_watch.init())

    @patch.object(pid_settings_system_api, "upsert")
    @patch.object(PidSettings, "__init__")
    @patch.object(PidSettings, "get")
    def test_pid_settings_upsert_failure_returns_none(
        self,
        mock_pid_settings_get,
        mock_pid_settings_init,
        mock_pid_settings_upsert,
    ):
        """test_pid_settings_upsert_failure_returns_none"""

        mock_pid_settings_get.return_value = False
        mock_pid_settings_init.return_value = None
        mock_pid_settings_upsert.side_effect = Exception(
            "mock_pid_settings_upsert_exception"
        )

        self.assertIsNone(pid_settings_watch.init())

    @patch.object(pid_settings_system_api, "upsert")
    @patch.object(PidSettings, "__init__")
    @patch.object(PidSettings, "get")
    def test_pid_settings_get_false_returns_none(
        self,
        mock_pid_settings_get,
        mock_pid_settings_init,
        mock_pid_settings_upsert,
    ):
        """test_pid_settings_get_false_returns_none"""

        mock_pid_settings_get.return_value = False
        mock_pid_settings_init.return_value = None
        mock_pid_settings_upsert.return_value = None

        self.assertIsNone(pid_settings_watch.init())
